#include "core/Image.hpp"

#include <cerrno>
#include <core/Logger.hpp>
#include <core/Utility.hpp>
#include <cstring>

#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wpragmas"
#pragma GCC diagnostic ignored "-Wunknown-warning-option"
#pragma GCC diagnostic ignored "-Wmaybe-uninitialized"
#pragma GCC diagnostic ignored "-Walloc-zero"
#define STB_IMAGE_IMPLEMENTATION
#define STBI_ONLY_PNG
#define STBI_MALLOC(n) Core::allocateRaw(n)
#define STBI_REALLOC(p, n) Core::reallocateRaw(p, n)
#define STBI_FREE(p) Core::deallocateRaw(p)
#include <stb_image.h>
#pragma GCC diagnostic pop

static void reportError(const char* path) {
    REPORT(
        Core::LogLevel::ERROR, "Cannot read image '#': #", path,
        strerror(errno));
}

bool Core::Image8::read(const char* path) {
    auto d = stbi_load(path, &width, &height, &channels, 0);
    if(d == nullptr || width < 0 || height < 0 || channels < 0) {
        reportError(path);
        return true;
    }
    size_t bytes = static_cast<size_t>(width * height * channels);
    data.resize(bytes);
    memcpy(data.begin(), d, bytes);
#ifdef CHECK_MEMORY
    memset(d, 0, bytes); // canary check with lib allocated memory
#endif
    stbi_image_free(d);
    return false;
}

bool Core::Image16::read(const char* path) {
    auto d = stbi_load_16(path, &width, &height, &channels, 0);
    if(d == nullptr || width < 0 || height < 0 || channels < 0) {
        reportError(path);
        return true;
    }
    size_t length = static_cast<size_t>(width * height * channels);
    data.resize(length);
    memcpy(data.begin(), d, length * 2);
#ifdef CHECK_MEMORY
    memset(d, 0, length * 2); // canary check with lib allocated memory
#endif
    stbi_image_free(d);
    return false;
}
